package edu.iastate.cs228.hw1;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Scanner;
import java.util.InputMismatchException;

public class BattlefieldClient {
	public static <T extends MilitaryTransport> void main(String[] args)
			throws BattlefieldException {
		Battlefield<T> battlefield = null;
		playGame(battlefield);

	}

	private static <T extends MilitaryTransport> void playGame(Battlefield<T> bf)
			throws BattlefieldException {
		Scanner userInput = new Scanner(System.in);
		bf = enterSizeofBattlefield(bf);
		while (true) {
			System.out
					.println("Please enter a MilitaryTransport in forms of type, name, initial defense or start the game");
			String text = userInput.next();
			if (startGame(text)) {
				break;
			}
			addMilitatyTransport(bf, text);
		}
		simulateBattleField(bf);
		System.out.println("Do you want to simulate again? Y/N");
		String text = userInput.next();
		if (text.equals("Y")) {
			playGame(bf);
		} else if (text.equals("N")) {
			userInput.close();
		}
	}

	private static <T extends MilitaryTransport> void simulateBattleField(
			Battlefield<T> bf) throws BattlefieldException {
		bf.simulate();
		String[] transports = bf.listTransportsSorted();
		for (int i = 0; i < transports.length; i++) {
			System.out.println(transports[i]);
		}
	}

	private static <T extends MilitaryTransport> void addMilitatyTransport(
			Battlefield<T> bf, String text) throws BattlefieldException {
		if (isFormatCorrect(text) && isTypeOfTransportValid(text)) {
			MilitaryTransport mt = null;
			mt = parseInputToMilitaryTransport(text, mt);
			addWithinCapacity(mt, bf);
		} else {
			System.out
					.println("The format or type is not correct, please enter again");
			Scanner sc = new Scanner(System.in);
			String input = sc.next();
			addMilitatyTransport(bf, input);
		}
	}

	private static <T extends MilitaryTransport> Battlefield enterSizeofBattlefield(
			Battlefield<T> bf)  {
		System.out.println("Please enter the size of the battle field");
		Scanner input = new Scanner(System.in);
		try {
			int size = input.nextInt();
			bf = new Battlefield<T>(size);
			return bf;
		} 
		
		catch(InputMismatchException e){
			System.out.println("Please enter a number");
			enterSizeofBattlefield(bf);
		}
		
		catch (BattlefieldException e) {
			System.out.println("Please try to enter another number");
			enterSizeofBattlefield(bf);
		}
		return bf;
	}

	public static MilitaryTransport parseInputToMilitaryTransport(String input,
			MilitaryTransport mt) {
		String[] mtAttributes = new String[3];
		parseInputToArray(input, mtAttributes);
		if(mtAttributes[0] == null){
			System.out
			.println("The format is not correct, please enter again");
			Scanner sc = new Scanner(System.in);
			String newInput = sc.next();
			parseInputToMilitaryTransport(newInput,mt);
		}
		
		try {
			int defense = Integer.parseInt(mtAttributes[2]);
			if (mtAttributes[0].equals("ArmoredVehicle")) {
				mt = new ArmoredVehicle(mtAttributes[1], defense);
				return mt;
			} else if (mtAttributes[0].equals("Tank")) {
				mt = new Tank(mtAttributes[1], defense);
				return mt;
			} else if (mtAttributes[0].equals("Jeep")) {
				mt = new Jeep(mtAttributes[1], defense);
				return mt;
			} else {
				return mt;
			}
		} catch (NumberFormatException e) {
			return mt;
		}

	}

	public static void parseInputToArray(String input, String[] targetArray) {
		String[] arrayOfInputs = input.split(",");
		
		if(arrayOfInputs.length != targetArray.length){
			return;
		}
		else{
			for (int i = 0; i < targetArray.length; i++) {
				targetArray[i] = arrayOfInputs[i];
			}
		}
		
		
	}

	public static boolean isTypeOfTransportValid(String input) {
		String[] inputs = new String[3];
		parseInputToArray(input, inputs);
		if (inputs[0].equals("ArmoredVehicle") || inputs[0].equals("Jeep")
				|| inputs[0].equals("Tank")) {
			return true;
		} else {
			return false;
		}
	}

	public static boolean isFormatCorrect(String input) {
		String[] inputs = new String[3];
		parseInputToArray(input, inputs);
		for(String e:inputs){
			if(e == null){
				return false;
			}
		}

		try {
			int defense = Integer.parseInt(inputs[2]);
		} catch (NumberFormatException e) {
			return false;
		}

		return true;
	}

	public static boolean startGame(String input) {
		if (input.equals("s")) {
			return true;
		}
		return false;
	}

	public static <T extends MilitaryTransport> void addWithinCapacity(
			MilitaryTransport mt, Battlefield<T> bf)
			throws BattlefieldException {
		int current = bf.listTransportsSorted().length;
		if (bf.getCapacity() == current) {
			System.out.println("Please Start the Game");
			return;
		} else {
			if ((duplicateName(bf, mt) == false)) {
				bf.add((T) mt);
			} else {
				System.out.println("Please Enter Another Name");
				return;
			}

		}
	}

	public static <T extends MilitaryTransport> boolean duplicateName(
			Battlefield<T> b, MilitaryTransport m) {
		String[] allTransports = b.listTransportsSorted();
		if (allTransports.length == 0) {
			return false;
		}
		for (String e : allTransports) {
			String[] mt = e.split(",");
			if (mt[0].equals(m.getName())) {
				return true;
			}
		}
		return false;

	}
}
